@@ -1,70 +1,87 @@
 package CPAN::WWW::Testers;
+
+use strict;
+use vars qw($VERSION);
+
+$VERSION = "0.34";
+
+#----------------------------------------------------------------------------
+# Library Modules
+
+use Archive::Extract;
 use DateTime;
 use DBI;
 use File::Copy;
+use File::Path;
 use File::stat;
 use File::Slurp;
 use JSON::Syck;
 use LWP::Simple;
 use Parse::BACKPAN::Packages;
+use Parse::CPAN::Distributions;
 use Path::Class;
 use Template;
+use Sort::Versions;
 use Storable qw(dclone);
 use XML::RSS;
 use YAML;
-use strict;
-use vars qw($VERSION);
-use version;
+
 use base qw(Class::Accessor::Chained::Fast);
-__PACKAGE__->mk_accessors(qw(directory database dbh tt last_id backpan));
-$VERSION = "0.33";
+
+#----------------------------------------------------------------------------
+# Variables
+
+my $DEFAULT_URL = 'http://devel.cpantesters.org/cpanstats.db.gz';
+my $DEFAULT_DB  = './cpanstats.db';
+
+use constant RSS_LIMIT_RECENT => 200;
+use constant RSS_LIMIT_AUTHOR => 100;
+
+#----------------------------------------------------------------------------
+# The Application Programming Interface
+
+__PACKAGE__->mk_accessors(qw(directory database dbh tt last_id backpan oncpan updates list));
+
+sub download {
+    my $self    = shift;
+    my $source  = shift || $DEFAULT_URL;
+    my $file  = basename($source);
+
+    my $target = file( $self->directory, $file );
+    mirror( $source, $target );
+
+    #system("bunzip -kf $target");
+    my $ae = Archive::Extract->new( archive => $target );
+    unless($ae->extract( to => $self->directory )) {
+        die 'Failed to extract the archive [$target]';
+    }
+
+    my @files = $ae->files();
+    $self->database( $self->directory, $files[0] );
+}
 
 sub generate {
     my $self = shift;
 
-    $self->download;
-    $self->write;
-}
+    # ensure we have a database
+    my $db = $self->database;
+    $db = $DEFAULT_DB           unless($db && -f $db);
+    die 'No database found!\n'  unless($db && -f $db);
 
-sub _last_id {
-    my ( $self, $id ) = @_;
-    my $filename = file( $self->directory, "last_id.txt" )->stringify;
-
-    overwrite_file( $filename, 0 ) unless -f $filename;
-
-    if ($id) {
-        overwrite_file( $filename, $id );
-    } else {
-        my $id = read_file($filename);
-        return $id;
-    }
-}
-
-sub download {
-    my $self = shift;
-
-    my $url = "http://www.astray.com/tmp/testers.db";
-    my $file = file( $self->directory, "testers.db" );
-    mirror( $url, $file );
-    $self->database( $self->directory );
-}
-
-sub write {
-    my $self = shift;
-
-    my $db = file( $self->database, "testers.db" );
     my $dbh = DBI->connect( "dbi:SQLite:dbname=$db", '', '',
         { RaiseError => 1 } );
     $self->dbh($dbh);
 
     my $backpan = Parse::BACKPAN::Packages->new();
     $self->backpan($backpan);
+    my $oncpan = Parse::CPAN::Distributions->new(file => $self->list);
+    $self->oncpan($oncpan);
 
     my $directory = $self->directory;
 
+    # set up API to Template Toolkit
     my $tt = Template->new(
         {
-
             #    POST_CHOMP => 1,
             #    PRE_CHOMP => 1,
             #    TRIM => 1,
@@ -82,8 +99,9 @@ sub write {
     );
     $self->tt($tt);
 
+    # generate pages
     $self->_copy_files;
-    $self->_write_alphabetic;
+    $self->_write_distributions_alphabetic;
     $self->_write_distributions;
     $self->_write_authors_alphabetic;
     $self->_write_authors;
@@ -92,36 +110,121 @@ sub write {
     $self->_write_index;
 }
 
+sub update {
+    my $self = shift;
+
+    # ensure we have a database
+    my $db = $self->database;
+    $db = $DEFAULT_DB           unless($db && -f $db);
+    die 'No database found!\n'  unless($db && -f $db);
+
+    my $dbh = DBI->connect( "dbi:SQLite:dbname=$db", '', '',
+        { RaiseError => 1 } );
+    $self->dbh($dbh);
+
+    my $backpan = Parse::BACKPAN::Packages->new();
+    $self->backpan($backpan);
+    my $oncpan = Parse::CPAN::Distributions->new(file => $self->list);
+    $self->oncpan($oncpan);
+
+    my $directory = $self->directory;
+
+    # set up API to Template Toolkit
+    my $tt = Template->new(
+        {
+            #    POST_CHOMP => 1,
+            #    PRE_CHOMP => 1,
+            #    TRIM => 1,
+            EVAL_PERL    => 1,
+            INCLUDE_PATH => [ 'src', "$directory/templates" ],
+            PROCESS      => 'layout',
+            FILTERS      => {
+                'striphtml' => sub {
+                    my $text = shift;
+                    $text =~ s/<.+?>//g;
+                    return $text;
+                },
+            },
+        }
+    );
+    $self->tt($tt);
+
+    my (@dists,@authors);
+    my $updates = $self->updates;
+    my $fh = IO::File->new($updates,'r') or die "Cannot open updates file [$updates]: $!\n";
+    while(<$fh>) {
+        my ($name,$value) = split(':');
+        $value =~ s/\s+$//;
+        push @dists,   $value   if($name eq 'dist');
+        push @authors, $value   if($name eq 'author');
+    }
+
+    # generate pages
+    $self->_copy_files;
+    if(@dists) {
+        $self->_write_distributions_alphabetic;
+        $self->_write_distributions(@dists);
+    }
+    if(@authors) {
+        $self->_write_authors_alphabetic;
+        $self->_write_authors(@authors);
+    }
+    $self->_write_recent;
+    $self->_write_stats;
+    $self->_write_index;
+}
+
+sub _last_id {
+    my ( $self, $id ) = @_;
+    my $filename = file( $self->directory, "last_id.txt" )->stringify;
+
+    overwrite_file( $filename, 0 ) unless -f $filename;
+
+    if ($id) {
+        overwrite_file( $filename, $id );
+    } else {
+        $id = read_file($filename);
+    }
+
+    return $id;
+}
+
 sub _copy_files {
     my $self      = shift;
     my $dbh       = $self->dbh;
     my $directory = $self->directory;
-    my $tt        = $self->tt;
 
     foreach my $filename (
-        'style.css', 'red.png', 'yellow.png', 'green.png',
-        'background.png'
+        'style.css',
+        'cssrules.js',
+        'red.png', 'yellow.png', 'green.png', 'background.png'
         )
     {
-        my $src = file( 'src', $filename );
-        my $dest = file( $directory, $filename );
+        my $src  = "src/$filename";
+        my $dest = "$directory/$filename";
         copy( $src, $dest );
     }
+
+    my $dir = dir( $directory, 'stats', 'dist' );
+    mkpath("$dir");
+    die $!  unless(-d "$dir");
 }
 
-sub _write_alphabetic {
+sub _write_distributions_alphabetic {
     my $self      = shift;
     my $dbh       = $self->dbh;
     my $directory = $self->directory;
     my $now       = DateTime->now;
     my $tt        = $self->tt;
 
-    mkdir dir( $directory, 'letter' ) || die $!;
+    my $dir = dir( $directory, 'letter' );
+    mkpath("$dir");
+    die $!  unless(-d "$dir");
 
     foreach my $letter ( 'A' .. 'Z' ) {
         my $dist;
         my $sth = $dbh->prepare(
-            "SELECT DISTINCT(distribution) FROM reports where distribution like ?"
+            "SELECT DISTINCT(dist) FROM cpanstats WHERE dist LIKE ?"
         );
         $sth->execute("$letter%");
         $sth->bind_columns( \$dist );
@@ -150,7 +253,9 @@ sub _write_authors_alphabetic {
     my $now       = DateTime->now;
     my $tt        = $self->tt;
 
-    mkdir dir( $directory, 'lettera' ) || die $!;
+    my $dir = dir( $directory, 'lettera' );
+    mkpath("$dir");
+    die $!  unless(-d "$dir");
 
     my @all_authors = $backpan->authors;
 
@@ -175,75 +280,114 @@ sub _write_authors {
     my $directory = $self->directory;
     my $last_id   = $self->_last_id;
     my $backpan   = $self->backpan;
+    my $oncpan    = $self->oncpan;
     my $now       = DateTime->now;
     my $tt        = $self->tt;
+    my $count     = 0;
 
-    mkdir dir( $directory, 'letter' ) || die $!;
+    my $dir = dir( $directory, 'letter' );
+    mkpath("$dir");
+    die $!  unless(-d "$dir");
 
-    # we only want to update authors that have had changes from our
-    # last update
-    my $dist_sth = $dbh->prepare(
-        "SELECT DISTINCT(distribution) FROM reports WHERE id > $last_id");
-    my $distribution;
-    $dist_sth->execute;
-    $dist_sth->bind_columns( \$distribution );
+    my @authors;
+    if(@_) {
+        @authors = @_;
 
-    my $author_of;
-    foreach my $author ( $backpan->authors ) {
-        foreach my $distribution ( $backpan->distributions_by($author) ) {
-            $author_of->{$distribution} = $author;
+    } else {
+        my $sth = $dbh->prepare(
+            "SELECT count(id) FROM cpanstats WHERE id > $last_id");
+        $sth->execute;
+        $sth->bind_columns( \$count );
+        $sth->fetch;
+        if($count > 100000) {
+            # rebuild for all authors if we're looking at a large number
+            # of reports, as checking backpan for distributions is EXTREMELY
+            # time consuming! There are less than 4000 authors in total anyway.
+            @authors = $backpan->authors;
+        } else {
+            # if only updating for a smaller selection of reports, only update
+            # for those authors that have had reports since our last update
+            my $sth = $dbh->prepare(
+                "SELECT dist,version FROM cpanstats WHERE id > $last_id GROUP BY dist,version");
+            my ($distribution,$version);
+            $sth->execute;
+            $sth->bind_columns( \$distribution, \$version );
+
+            my %authors;
+            while ( $sth->fetch ) {
+                #print "... checking distro=$distribution-$version\n";
+                my $author = $oncpan->author_of($distribution,$version);
+                if($author) {
+                    $authors{$author}++;
+                } else {
+                    foreach my $dist ( $backpan->distributions($distribution) ) {
+                    #print "... version=".$dist->{version}." [$version]\n";
+                    #print "... author=".$dist->{cpanid}."\n";
+                        if($dist->{version} eq $version) {
+                            $authors{$dist->{cpanid}}++;
+                            last;
+                        }
+                    }
+                }
+            }
+            @authors = keys %authors;
         }
     }
 
-    my %authors;
-    while ( $dist_sth->fetch ) {
-        my $author = $author_of->{$distribution};
-        next unless $author;
-        $authors{$author}++;
-    }
-    my @authors = sort keys %authors;
+    print "Updating ".(scalar(@authors))." authors, from $count entries\n";
 
-    foreach my $author (@authors) {
-        my @distributions;
-        foreach my $distribution ( $backpan->distributions_by($author) ) {
+    foreach my $author (sort @authors) {
+        print "Processing $author\n";
+        my (%distributions,@distributions);
+
+        # if an author has no entries in BACKPAN, the next line can blow up!
+        eval{ %distributions = map {$_ => 1} $backpan->distributions_by($author), $oncpan->distributions_by($author); };
+        eval{ %distributions = map {$_ => 1} $oncpan->distributions_by($author); }  if($@);
+
+        foreach my $distribution (sort keys %distributions ) {
             next unless $distribution =~ /^[A-Za-z0-9][A-Za-z0-9-_]+$/;
+            #print "... dist $distribution\n";
 
-            my $latest_version;
-            my $latest_sth = $dbh->prepare(
-                "select version from reports where distribution = '$distribution' order by id desc limit 1"
-            );
-            $latest_sth->execute;
-            $latest_sth->bind_columns( \$latest_version );
-            $latest_sth->fetch;
+            # Note that we assume CPAN holds the latest version. It's possible
+            # that this is not the case, but hopefully that is a short term
+            # anomaly
+            my $latest_version = $oncpan->latest_version($distribution,$author);
 
-            my $sth = $dbh->prepare( "
-SELECT id, status, perl, osname, osvers, archname FROM reports 
-WHERE distribution = ? and version = ? order by id
-" );
+            # If no version found on CPAN, check BACKPAN
+            unless($latest_version) {
+                foreach my $dist ( sort {versioncmp($b->{version},$a->{version})} $backpan->distributions($distribution) ) {
+                    if($dist->{cpanid} eq $author) {
+                        $latest_version = $dist->{version};
+                        last;
+                    }
+                }
+            }
+
+            my $sth = $dbh->prepare(
+                "SELECT id, state, perl, osname, osvers, platform FROM cpanstats WHERE dist = ? AND version = ? AND state != 'cpan' ORDER BY id" );
             $sth->execute( $distribution, $latest_version );
             my ( $id, $status, $perl, $osname, $osvers, $archname );
             $sth->bind_columns( \$id, \$status, \$perl, \$osname, \$osvers,
                 \$archname );
-            my @reports;
+            my (@reports,$summary);
             while ( $sth->fetch ) {
                 my $report = {
                     id           => $id,
                     distribution => $distribution,
-                    status       => $status,
+                    status       => uc $status,
                     version      => $latest_version,
                     perl         => $perl,
                     osname       => $osname,
                     osvers       => $osvers,
                     archname     => $archname,
-                    url =>
-                        "http://nntp.x.perl.org/group/perl.cpan.testers/$id",
+                    url          => "http://nntp.x.perl.org/group/perl.cpan.testers/$id",
+                    csspatch     => $perl =~ /patch/ ? 'patch' : 'regular',
+                    cssperl      => $perl =~ /^5.(7|9|11)/ ? 'perldev' : 'perlfull',
                 };
                 push @reports, $report;
-            }
 
-            my ($summary);
-            foreach my $report (@reports) {
                 $summary->{ $report->{status} }++;
+                $summary->{ 'ALL' }++;
             }
 
             push @distributions,
@@ -252,6 +396,8 @@ WHERE distribution = ? and version = ? order by id
                 version      => $latest_version,
                 reports      => \@reports,
                 summary      => $summary,
+                csscurrent   => $self->oncpan->listed($distribution,$latest_version) ? 'oncpan' : 'backpan',
+                cssrelease   => $latest_version =~ /_/ ? 'develrel' : 'official',
                 };
         }
 
@@ -272,11 +418,17 @@ WHERE distribution = ? and version = ? order by id
             push @reports, @{ $distribution->{reports} };
         }
         @reports = sort { $b->{id} <=> $a->{id} } @reports;
+        splice(@reports,RSS_LIMIT_AUTHOR);
 
         $destfile = file( $directory, 'author', $author . ".rss" );
         print "Writing $destfile\n";
         overwrite_file( $destfile->stringify,
             _make_rss_author( $author, \@reports ) );
+
+        $destfile = file( $directory, 'author', $author . "-nopass.rss" );
+        print "Writing $destfile\n";
+        overwrite_file( $destfile->stringify,
+            _make_rss_author_nopass( $author, \@reports ) );
     }
 }
 
@@ -286,6 +438,7 @@ sub _write_distributions {
     my $directory = $self->directory;
     my $last_id   = $self->_last_id;
     my $backpan   = $self->backpan;
+    my $oncpan    = $self->oncpan;
     my $now       = DateTime->now;
     my $tt        = $self->tt;
 
@@ -293,24 +446,30 @@ sub _write_distributions {
     # last update
     my @distributions;
 
-    my $dist_sth = $dbh->prepare(
-        "SELECT DISTINCT(distribution) FROM reports WHERE id > $last_id");
-    my $distribution;
-    $dist_sth->execute;
-    $dist_sth->bind_columns( \$distribution );
-    while ( $dist_sth->fetch ) {
-        push @distributions, $distribution;
+    if(@_) {
+        @distributions = @_;
+    } else {
+        my $sth = $dbh->prepare(
+            "SELECT DISTINCT(dist) FROM cpanstats WHERE id > $last_id");
+        my $distribution;
+        $sth->execute;
+        $sth->bind_columns( \$distribution );
+        while ( $sth->fetch ) {
+            push @distributions, $distribution;
+        }
     }
-    @distributions = sort @distributions;
+
+    print "Updating ".(scalar(@distributions))." distributions\n";
 
     # process distribution pages
-    foreach my $distribution (@distributions) {
+    foreach my $distribution (sort @distributions) {
         next unless $distribution =~ /^[A-Za-z0-9][A-Za-z0-9-_]+$/;
+        print "Processing $distribution\n";
 
-        my $action_sth = $dbh->prepare( "
-SELECT id, status, version, perl, osname, osvers, archname FROM reports 
-WHERE distribution = ? order by id
-" );
+        #print STDERR "DEBUG:dist=[$distribution]\n";
+
+        my $action_sth = $dbh->prepare(
+            "SELECT id, state, version, perl, osname, osvers, platform FROM cpanstats WHERE dist = ? AND state != 'cpan' ORDER BY version, id" );
         $action_sth->execute($distribution);
         my ( $id, $status, $version, $perl, $osname, $osvers, $archname );
         $action_sth->bind_columns(
@@ -319,24 +478,31 @@ WHERE distribution = ? order by id
         );
         my @reports;
         while ( $action_sth->fetch ) {
+            #print STDERR "DEBUG:report:id=[$id],status=[$status],version=[$version]\n";
             next unless $version;
+            $perl = "5.004_05" if $perl eq "5.4.4"; # RT 15162
             my $report = {
                 id           => $id,
                 distribution => $distribution,
-                status       => $status,
+                status       => uc $status,
                 version      => $version,
                 perl         => $perl,
                 osname       => $osname,
                 osvers       => $osvers,
                 archname     => $archname,
-                url => "http://nntp.x.perl.org/group/perl.cpan.testers/$id",
+                url          => "http://nntp.x.perl.org/group/perl.cpan.testers/$id",
+                csspatch     => $perl =~ /patch/ ? 'patch' : 'regular',
+                cssperl      => $perl =~ /^5.(7|9|11)/ ? 'perldev' : 'perlfull',
             };
             push @reports, $report;
         }
 
+        #print STDERR "DEBUG:count:".(scalar(@reports))."\n";
+
         my ( $summary, $byversion );
         foreach my $report (@reports) {
             $summary->{ $report->{version} }->{ $report->{status} }++;
+            $summary->{ $report->{version} }->{ 'ALL' }++;
             push @{ $byversion->{ $report->{version} } }, $report;
         }
 
@@ -346,35 +512,44 @@ WHERE distribution = ? order by id
                 = [ sort { $b->{id} <=> $a->{id} } @reports ];
         }
 
-        my @versions = sort {
-            my $c;
-            eval { $c = version->new($b) <=> version->new($a) };
-            $@ ? ( $b <=> $a ) : $c
-            } map {
-            $_->version
-            } $backpan->distributions($distribution);
+        #print STDERR "BY  :versions:".(join(",",(keys %$byversion)))."\n";
+        #print STDERR "CPAN:versions:".(join(",",($oncpan->versions($distribution))))."\n";
+        #print STDERR "BACK:versions:".(join(",",(map {$_->version} $backpan->distributions($distribution))))."\n";
 
-        my $stats;
-        my $sth = $dbh->prepare( "
-SELECT perl, osname, count(*) FROM reports
-WHERE distribution = ?
-GROUP BY perl, osname" );
+        # ensure we cover all known versions
+        my %versions = map {$_ => 1}
+                            keys %$byversion,
+                            $oncpan->versions($distribution),
+                            map {$_->version} $backpan->distributions($distribution);
+
+        my @versions = sort {versioncmp($b,$a)} keys %versions;
+
+        my %release;
+        foreach my $version ( keys %versions ) {
+            $release{$version}->{csscurrent} = $oncpan->listed($distribution,$version) ? 'oncpan' : 'backpan';
+            $release{$version}->{cssrelease} = $version =~ /_/ ? 'develrel' : 'official';
+        }
+
+        #print STDERR "DEBUG:backpan:".(scalar(keys %versions))."\n";
+        #print STDERR "DEBUG:versions:".(join(",",(@versions)))."\n";
+
+        my ($stats,$oses);
+        my $sth = $dbh->prepare(
+            "SELECT perl, osname, count(*) FROM cpanstats WHERE dist = ? GROUP BY perl, osname" );
         $sth->execute($distribution);
         while ( my ( $perl, $osname, $count ) = $sth->fetchrow_array ) {
-
-            #    warn "$perl $osname $count\n";
+            # warn "$perl $osname $count\n";
             $stats->{$perl}->{$osname} = $count;
+            $oses->{$osname} = 1;
         }
-        my $oses;
-        foreach my $perl ( keys %$stats ) {
-            $oses->{$_} = 1 foreach keys %{ $stats->{$perl} };
-        }
+
         my @stats_oses = sort keys %$oses;
-        my @stats_perl = sort keys %$stats;
+        my @stats_perl = sort {versioncmp($a,$b)} keys %$stats;
 
         my $parms = {
             versions       => \@versions,
             summary        => $summary,
+            release        => \%release,
             byversion      => $byversion,
             distribution   => $distribution,
             now            => $now,
@@ -391,6 +566,8 @@ GROUP BY perl, osname" );
         print "Writing $destfile\n";
         overwrite_file( $destfile->stringify,
             _make_yaml_distribution( $distribution, \@reports ) );
+
+        splice(@reports,RSS_LIMIT_AUTHOR);
         $destfile = file( $directory, 'show', $distribution . ".rss" );
         print "Writing $destfile\n";
         overwrite_file( $destfile->stringify,
@@ -399,6 +576,19 @@ GROUP BY perl, osname" );
         print "Writing $destfile\n";
         overwrite_file( $destfile->stringify,
             _make_json_distribution( $distribution, \@reports ) );
+
+        # distribution PASS stats
+        $sth = $dbh->prepare(
+            "SELECT perl, osname, version FROM cpanstats WHERE dist = ? AND state='pass'" );
+        $sth->execute($distribution);
+        while ( my ( $perl, $osname, $version ) = $sth->fetchrow_array ) {
+            # warn "$perl $osname $version\n";
+            $stats->{$perl}->{$osname} = $version   if(!$stats->{$perl}->{$osname} || versioncmp($version,$stats->{$perl}->{$osname}));
+        }
+        $destfile = file( $directory, 'stats', 'dist', $distribution . ".html" );
+        print "Writing $destfile\n";
+        $tt->process( 'stats-dist', $parms, $destfile->stringify )
+            || die $tt->error;
     }
 }
 
@@ -409,21 +599,15 @@ sub _write_stats {
     my $tt        = $self->tt;
     my $now       = DateTime->now;
 
-    mkdir dir( $directory, 'stats' ) || die $!;
+    my $dir = dir( $directory, 'stats' );
+    mkpath("$dir");
+    die $!  unless(-d "$dir");
 
     my $limit;    # set to a small number for debugging only
-
-    my %data;
-    my %perldata;
-    my %perls;
-    my %all_osnames;
-    my %dists;
-    my %perlos;
+    my (%data,%perldata,%perls,%all_osnames,%dists,%perlos);
 
     my $sth = $dbh->prepare(
-        "SELECT distribution, version, perl, osname 
-                    FROM reports WHERE status = 'PASS'"
-    );
+        "SELECT dist, version, perl, osname FROM cpanstats WHERE state = 'pass'");
     $sth->execute;
     no warnings( 'uninitialized', 'numeric' );
     my $cnt;
@@ -432,9 +616,10 @@ sub _write_stats {
         next if not $perl;
         next if $perl =~ / /;
         next if $perl =~ /^5\.7/;
-        next if $perl =~ /^5\.9/;
+        #next if $perl =~ /^5\.9/;
 
         next if $version =~ /[^\d.]/;
+        $perl = "5.004_05" if $perl eq "5.4.4"; # RT 15162
 
         last if $limit and $cnt++ > $limit;
         $perldata{$perl}{$dist} = $version
@@ -448,8 +633,14 @@ sub _write_stats {
     }
     $sth->finish;
 
+    my @versions = sort {versioncmp($b,$a)} keys %perls;
+#        map {$_->{external}}
+#        sort {$b->{internal} <=> $a->{internal}}
+#        map {my $v = version->new($_); {internal => $v->numify, external => $_}} keys %perls;
+
     # page perl perl version cross referenced with platforms
-    foreach my $perl ( keys %perldata ) {
+    my %perl_osname_all;
+    foreach my $perl ( @versions ) {
         my @data;
         my %oscounter;
         my %dist_for_perl;
@@ -475,6 +666,7 @@ sub _write_stats {
         foreach my $os ( sort keys %{ $perlos{$perl} } ) {
             if ( $oscounter{$os} ) {
                 push @perl_osnames, { os => $os, cnt => $oscounter{$os} };
+                $perl_osname_all{$os}{$perl} = $oscounter{$os};
             }
         }
 
@@ -487,20 +679,49 @@ sub _write_stats {
             perl        => $perl,
             cnt_modules => scalar keys %dist_for_perl,
         };
+        print "Writing $destfile\n";
         $tt->process( "stats-perl-platform", $parms, $destfile->stringify )
             || die $tt->error;
     }
 
+    # how many test reports per platform per perl version?
+    {
+        my @data;
+        my @perl_osnames = map {{os => $_}} keys %perl_osname_all;
+
+        foreach my $perl ( @versions ) {
+            my @count;
+            foreach my $os (keys %perl_osname_all) {
+                push @count, { os => $os, count => $perl_osname_all{$os}{$perl} };
+            }
+            push @data, {
+                perl => $perl,
+                count => \@count,
+            }
+        }
+
+        my $destfile
+            = file( $directory, 'stats', "perl_platforms.html" );
+        my $parms = {
+            now         => $now,
+            osnames     => \@perl_osnames,
+            perlv       => \@data,
+        };
+        print "Writing $destfile\n";
+        $tt->process( "stats-perl-platform-count", $parms, $destfile->stringify )
+            || die $tt->error;
+    }
+
     # page per perl version
-    foreach my $perl ( keys %perldata ) {
+    foreach my $perl ( @versions ) {
         my @data;
         my $cnt;
         foreach my $dist ( sort keys %{ $perldata{$perl} } ) {
             $cnt++;
             push @data,
                 {
-                dist    => $dist,
-                version => $perldata{$perl}{$dist},
+                    dist    => $dist,
+                    version => $perldata{$perl}{$dist},
                 };
         }
 
@@ -511,13 +732,14 @@ sub _write_stats {
             perl        => $perl,
             cnt_modules => $cnt,
         };
+        print "Writing $destfile\n";
         $tt->process( "stats-perl-version", $parms, $destfile->stringify )
             || die $tt->error;
     }
 
     # generate index.html
     my @perls;
-    foreach my $p ( sort keys %perls ) {
+    foreach my $p ( @versions ) {
         unshift @perls,
             {
             perl         => $p,
@@ -530,6 +752,7 @@ sub _write_stats {
         now   => $now,
         perls => \@perls,
     };
+        print "Writing $destfile\n";
     $tt->process( "stats-index", $parms, $destfile->stringify )
         || die $tt->error;
 }
@@ -543,18 +766,16 @@ sub _write_recent {
 
     # Get the last id
     my $last_id;
-    my $last_sth = $dbh->prepare("SELECT max(id) FROM reports");
+    my $last_sth = $dbh->prepare("SELECT max(id) FROM cpanstats");
     $last_sth->execute;
     $last_sth->bind_columns( \$last_id );
     $last_sth->fetch;
 
     # Recent reports
-    my $recent_id = $last_id - 200;
+    my $recent_id = $last_id - RSS_LIMIT_RECENT;
     my @recent;
-    my $recent_sth = $dbh->prepare( "
-SELECT id, status, distribution, version, perl, osname, osvers, archname FROM reports 
-WHERE id > $recent_id order by id desc
-" );
+    my $recent_sth = $dbh->prepare(
+        "SELECT id, state, dist, version, perl, osname, osvers, platform FROM cpanstats WHERE id > $recent_id AND state != 'cpan' ORDER BY id desc" );
     $recent_sth->execute();
     my ( $id, $status, $distribution, $version, $perl, $osname, $osvers,
         $archname );
@@ -566,7 +787,7 @@ WHERE id > $recent_id order by id desc
         my $report = {
             id           => $id,
             distribution => $distribution,
-            status       => $status,
+            status       => uc $status,
             version      => $version,
             perl         => $perl,
             osname       => $osname,
@@ -600,19 +821,33 @@ sub _write_index {
 
     # Finally, the front page
     my $total_reports;
-    my $sth = $dbh->prepare("SELECT count(*) from reports");
+    my $sth = $dbh->prepare("SELECT count(*) FROM cpanstats WHERE state in ('pass','fail','na','unknown')");
     $sth->execute;
     $sth->bind_columns( \$total_reports );
     $sth->fetch;
 
+    my $db = $self->database;
+
     my $destfile = file( $directory, "index.html" );
     print "Writing $destfile\n";
     my $parms = {
         now           => $now,
         letters       => [ 'A' .. 'Z' ],
         total_reports => $total_reports,
+        dbsize        => int((-s $db     )/1024/1024),
+        dbzipsize     => int((-s "$db.gz")/1024/1024),
     };
+
+    print STDERR "dbsize=[$parms->{dbsize}], dbzipsize=[$parms->{dbzipsize}], db=[$db]\n";
+
     $tt->process( "index", $parms, $destfile->stringify ) || die $tt->error;
+
+    # now add all the redirects
+    for my $dir (qw(author letter lettera show)) {
+        my $src  = "src/index.html";
+        my $dest = "$directory/$dir/index.html";
+        copy( $src, $dest );
+    }
 }
 
 sub _make_yaml_distribution {
@@ -652,8 +887,8 @@ sub _make_rss_distribution {
     my $rss = XML::RSS->new( version => '1.0' );
 
     $rss->channel(
-        title       => "$dist CPAN Testers reports",
-        link        => "http://testers.cpan.org/show/$dist.html",
+        title       => "$dist CPAN Testers Reports",
+        link        => "http://www.cpantesters.org/show/$dist.html",
         description => "Automated test results for the $dist distribution",
         syn         => {
             updatePeriod    => "daily",
@@ -684,7 +919,7 @@ sub _make_rss_recent {
 
     $rss->channel(
         title       => "Recent CPAN Testers reports",
-        link        => "http://testers.cpan.org/recent.html",
+        link        => "http://www.cpantesters.org/recent.html",
         description => "Recent CPAN Testers reports",
         syn         => {
             updatePeriod    => "daily",
@@ -699,7 +934,7 @@ sub _make_rss_recent {
                 "%s %s-%s %s on %s %s (%s)",
                 @{$test}{
                     qw( status distribution version perl osname osvers archname )
-                    }
+                }
             ),
             link =>
                 "http://nntp.x.perl.org/group/perl.cpan.testers/$test->{id}",
@@ -710,12 +945,13 @@ sub _make_rss_recent {
 }
 
 sub _make_rss_author {
-    my ( $author, $reports ) = @_;
+    my ( $author, $reports, $prefix ) = @_;
     my $rss = XML::RSS->new( version => '1.0' );
+    $prefix ||= '';
 
     $rss->channel(
-        title       => "Reports for distributions by $author",
-        link        => "http://testers.cpan.org/author/$author.html",
+        title       => "${prefix}Reports for distributions by $author",
+        link        => "http://www.cpantesters.org/author/$author.html",
         description => "Reports for distributions by $author",
         syn         => {
             updatePeriod    => "daily",
@@ -730,7 +966,7 @@ sub _make_rss_author {
                 "%s %s-%s %s on %s %s (%s)",
                 @{$report}{
                     qw( status distribution version perl osname osvers archname )
-                    }
+                }
             ),
             link =>
                 "http://nntp.x.perl.org/group/perl.cpan.testers/$report->{id}",
@@ -740,7 +976,13 @@ sub _make_rss_author {
     return $rss->as_string;
 }
 
-1;
+sub _make_rss_author_nopass {
+    my ( $author, $reports ) = @_;
+    my @nopass = grep { $_->{status} ne 'PASS' } @$reports;
+    _make_rss_author( $author, \@nopass, 'Failing ' );
+}
+
+q("QA Automation, so much to answer for!");
 
 __END__
 
@@ -752,27 +994,33 @@ CPAN::WWW::Testers - Present CPAN Testers data
 
   my $t = CPAN::WWW::Testers->new();
   $t->directory($directory);
+  if($download) { $t->download($download); }
+  else          { $t->database($database); }
   $t->generate;
 
 =head1 DESCRIPTION
 
-The distribution can present CPAN Testers data.
-cpan-testers is a group which was initially setup by Graham Barr and
-Chris Nandor. The objective of the group is to test as many of the
-distributions on CPAN as possible, on as many platforms as possible.
-The ultimate goal is to improve the portability of the distributions
-on CPAN, and provide good feedback to the authors.
+This distribution generates the CPAN Testers Reports website.
 
-CPAN Testers is really a mailing list with a web interface,
-testers.cpan.org. testers.cpan.org was painfully slow. I happened to
-be doing metadata stuff for Module::CPANTS. This is the result. It's
-alpha code, but using it anyone can host their CPAN Testers website.
+=head1 CPAN TESTERS
 
-Unpack the distribution and look at examples/generate.pl. Wait
-patiently. Send patches and better design.
+cpan-testers is a group, that was originaly setup by Graham Barr and Chris
+Nandor.
 
-At the moment I am running the output of this at
-http://testers.astray.com/
+The objective of the group is to test as many of the distributions available on
+CPAN as possible, on as many platforms as possible, with a variety of perl
+interpreters. The ultimate goal is to improve the portability of the
+distributions on CPAN, and provide good feedback to the authors.
+
+CPAN Testers began as a mailing list with a web interface (see the NNTP
+website - http://nntp.x.perl.org/group/perl.cpan.testers/). Leon Brocard began
+working on extracting metadata for use with the CPANTS, and ended up creating
+the Reports website. This code now allows you to create and host your very own
+CPAN Testers website, should you so choose.
+
+Unpack the distribution and look at examples/generate.pl, to understand how
+the site is generated. If you would like to send patches or report bugs,
+please use the RT system.
 
 =head1 INTERFACE
 
@@ -790,45 +1038,68 @@ Instatiates the object CPAN::WWW::Testers.
 
 =over
 
-=item 
+=item
 
 =item * directory
 
 Accessor to set/get the directory where the webpages are to be created.
-Note this also where the local copy of testers.db will reside.
-
-=item * generate
-
-Initiates the $obj->download and $obj->write method calls.
-
-=item * download
-
-Downloads the latest article updates from the NNTP server for the
-cpan-testers newgroup. Articles are then stored in the news.db
-SQLite database.
 
 =item * database
 
-Path to the SQLite database.
+Accessor to set/get the local path to the SQLite database.
 
-=item * write
+=item * download
 
-Reads the local copy of the testers.db, and creates the alphabetic 
-index, distribution and main index web pages, together with the
-YAML and RSS pages for each distribution.
+Downloads a remote copy of the SQLite database containing the latest article
+updates from the NNTP server for the cpan-testers newgroup. The path to the
+local copy of the database is then provided to the database accessor.
+
+=item * generate
+
+Reads the local copy of the SQLite database, and creates the alphabetic index,
+distribution and main index web pages, together with the YAML and RSS pages
+for each distribution.
+
+=item * update
+
+Given an updates file (pass via the constructor hash), will read through the
+file and update the requested distritbutions and authors only. This is to
+enable the update of specific pages, which may have got accidentally missed
+during a regular generate() call. See the 'bin/cpanreps-verify' program for
+further details.
 
 =back
 
+=head1 BUGS, PATCHES & FIXES
+
+There are no known bugs at the time of this release. However, if you spot a
+bug or are experiencing difficulties, that is not explained within the POD
+documentation, please send bug reports and patches to the RT Queue (see below).
+
+Fixes are dependant upon their severity and my availablity. Should a fix not
+be forthcoming, please feel free to (politely) remind me.
+
+RT: http://rt.cpan.org/Public/Dist/Display.html?Name=CPAN-WWW-Testers
+
 =head1 SEE ALSO
 
-CPAN::WWW::Testers::Generator
+L<CPAN::WWW::Testers::Generator>
+L<CPAN::Testers::WWW::Statistics>
+
+F<http://www.cpantesters.org/>,
+F<http://stats.cpantesters.org/>,
+F<http://wiki.cpantesters.org/>
 
 =head1 AUTHOR
 
-Leon Brocard <acme@astray.com>
+  Original author:    Leon Brocard <acme@astray.com>   200?-2008
+  Current maintainer: Barbie       <barbie@cpan.org>   2008-present
 
-=head1 LICENSE
+=head1 COPYRIGHT AND LICENSE
 
-This code is distributed under the same license as Perl.
+  Copyright (C) 2002-2008 Leon Brocard <acme@astray.com>
+  Copyright (C) 2008      Barbie <barbie@cpan.org>
 
+  This module is free software; you can redistribute it and/or
+  modify it under the same terms as Perl itself.
 
